@@ -11,7 +11,7 @@ use Carp();
 use English qw( -no_match_vars );
 use DirHandle();
 
-our $VERSION = 0.03;
+our $VERSION = 0.04;
 
 sub _SIZE_OF_TZ_HEADER                     { return 44 }
 sub _SIZE_OF_TRANSITION_TIME_V1            { return 4 }
@@ -762,7 +762,7 @@ sub _negative_time_local {
 }
 
 sub _get_isdst_gmtoff_abbr_converted_from_gmt_base {
-    my ( $self, $time, $convert_from_gmt ) = @_;
+    my ( $self, $time ) = @_;
     my ( $isdst, $gmtoff, $abbr );
     my $tz = $self->timezone();
     $self->_read_tzfile();
@@ -786,19 +786,33 @@ sub _get_isdst_gmtoff_abbr_converted_from_gmt_base {
             last TRANSITION_TIME;
         }
     }
+    my $first_standard_time_type;
+    if ( defined $self->{_tzdata}->{$tz}->{local_time_types}->[0] ) {
+        $first_standard_time_type =
+          $self->{_tzdata}->{$tz}->{local_time_types}->[0];
+    }
+  FIRST_STANDARD_TIME_TYPE:
+    foreach
+      my $local_time_type ( @{ $self->{_tzdata}->{$tz}->{local_time_types} } )
+    {
+        if ( $local_time_type->{isdst} ) {
+        }
+        else {
+            $first_standard_time_type = $local_time_type;
+            last FIRST_STANDARD_TIME_TYPE;
+        }
+    }
     my $offset_found;
     if (
            ( defined $first_transition_time )
-        && ( $self->{_tzdata}->{$tz}->{local_time_types}->[0] )
-        && ( $time < $first_transition_time +
-            $self->{_tzdata}->{$tz}->{local_time_types}->[0]->{gmtoff} )
+        && ($first_standard_time_type)
+        && ( $time <
+            $first_transition_time + $first_standard_time_type->{gmtoff} )
       )
     {
-        my $local_time_type =
-          $self->{_tzdata}->{$tz}->{local_time_types}->[0];
-        $gmtoff       = $local_time_type->{gmtoff};
-        $isdst        = $local_time_type->{isdst};
-        $abbr         = $local_time_type->{abbr};
+        $gmtoff       = $first_standard_time_type->{gmtoff};
+        $isdst        = $first_standard_time_type->{isdst};
+        $abbr         = $first_standard_time_type->{abbr};
         $offset_found = 1;
     }
     elsif ( !$transition_time_found ) {
@@ -822,19 +836,16 @@ sub _get_isdst_gmtoff_abbr_converted_from_gmt_base {
         $isdst  = $local_time_type->{isdst};
         $abbr   = $local_time_type->{abbr};
     }
-    elsif ( my $local_time_type =
-        $self->{_tzdata}->{$tz}->{local_time_types}->[0] )
-    {
-        my $local_time_type = $self->{_tzdata}->{$tz}->{local_time_types}->[0];
-        $gmtoff = $local_time_type->{gmtoff};
-        $isdst  = $local_time_type->{isdst};
-        $abbr   = $local_time_type->{abbr};
+    else {
+        $gmtoff = $first_standard_time_type->{gmtoff};
+        $isdst  = $first_standard_time_type->{isdst};
+        $abbr   = $first_standard_time_type->{abbr};
     }
     return ( $isdst, $gmtoff, $abbr );
 }
 
-sub _get_isdst_gmtoff_abbr {
-    my ( $self, $time, $convert_from_gmt ) = @_;
+sub _get_isdst_gmtoff_abbr_as_localtime {
+    my ( $self, $time ) = @_;
     my ( $isdst, $gmtoff, $abbr );
     my $tz = $self->timezone();
     $self->_read_tzfile();
@@ -853,15 +864,29 @@ sub _get_isdst_gmtoff_abbr {
             last TRANSITION_TIME;
         }
     }
+    my $first_standard_time_type;
+    if ( defined $self->{_tzdata}->{$tz}->{local_time_types}->[0] ) {
+        $first_standard_time_type =
+          $self->{_tzdata}->{$tz}->{local_time_types}->[0];
+    }
+  FIRST_STANDARD_TIME_TYPE:
+    foreach
+      my $local_time_type ( @{ $self->{_tzdata}->{$tz}->{local_time_types} } )
+    {
+        if ( $local_time_type->{isdst} ) {
+        }
+        else {
+            $first_standard_time_type = $local_time_type;
+            last FIRST_STANDARD_TIME_TYPE;
+        }
+    }
     my $offset_found;
     if (   ( defined $first_transition_time )
         && ( $time < $first_transition_time ) )
     {
-        my $local_time_type =
-          $self->{_tzdata}->{$tz}->{local_time_types}->[0];
-        $gmtoff       = $local_time_type->{gmtoff};
-        $isdst        = $local_time_type->{isdst};
-        $abbr         = $local_time_type->{abbr};
+        $gmtoff       = $first_standard_time_type->{gmtoff};
+        $isdst        = $first_standard_time_type->{isdst};
+        $abbr         = $first_standard_time_type->{abbr};
         $offset_found = 1;
     }
     elsif ( !$transition_time_found ) {
@@ -885,13 +910,10 @@ sub _get_isdst_gmtoff_abbr {
         $isdst  = $local_time_type->{isdst};
         $abbr   = $local_time_type->{abbr};
     }
-    elsif ( my $local_time_type =
-        $self->{_tzdata}->{$tz}->{local_time_types}->[0] )
-    {
-        my $local_time_type = $self->{_tzdata}->{$tz}->{local_time_types}->[0];
-        $gmtoff = $local_time_type->{gmtoff};
-        $isdst  = $local_time_type->{isdst};
-        $abbr   = $local_time_type->{abbr};
+    else {
+        $gmtoff = $first_standard_time_type->{gmtoff};
+        $isdst  = $first_standard_time_type->{isdst};
+        $abbr   = $first_standard_time_type->{abbr};
     }
     return ( $isdst, $gmtoff, $abbr );
 }
@@ -902,7 +924,8 @@ sub local_time {
         $time = time;
     }
 
-    my ( $isdst, $gmtoff, $abbr ) = $self->_get_isdst_gmtoff_abbr($time);
+    my ( $isdst, $gmtoff, $abbr ) =
+      $self->_get_isdst_gmtoff_abbr_as_localtime($time);
     $time += $gmtoff;
 
     if (wantarray) {
@@ -986,7 +1009,7 @@ sub _read_transition_times {
         @transition_times = unpack 'l>' . $timecnt, $buffer;
     }
     elsif ( $sizeof_transition_time == _SIZE_OF_TRANSITION_TIME_V2() ) {
-        eval { @transition_times = unpack 'q>' . $timecnt, $buffer; } or do {
+        eval { @transition_times = unpack 'q>' . $timecnt, $buffer; 1; } or do {
             require Math::Int64;
             @transition_times =
               map { Math::Int64::net_to_int64($_) } unpack '(a8)' . $timecnt,
@@ -1323,6 +1346,7 @@ sub _set_abbrs {
               substr $self->{_tzdata}->{$tz}->{time_zone_abbreviation_strings},
               $local_time_type->{abbrind};
         }
+        $local_time_type->{abbr} =~ s/\0+$//smx;
         $index += 1;
     }
     return;
@@ -1529,7 +1553,7 @@ There has been a file system error while opening the the designated path.  This
 
 =item C<< Failed to stat %s:%s >>
 
-There has been a file system error while doing a L<stat|perlfunc/"stat"> the the designated path.  This could be permissions related, or the timezone in question doesn't exist?
+There has been a file system error while doing a L<stat|perlfunc/"stat"> on the designated path.  This could be permissions related, or the timezone in question doesn't exist?
 
 =item C<< Failed to read %s from %s:%s >>
 
